var Util = {
	Translate: function() { return ''; }
};

var PURichText = {

    editors: new Hash(),
    onPasteHandler: function (id) {
        var editor = PURichText.editors.get(id);
        setTimeout(function () {
            // insert caret so we can restore cursor position
            editor.addContent('<em id="caret">...pasting...</em>');
            if (!editor.textarea.visible()) {
                editor.updateTextarea();
            }
            editor.cleanHTML();
            editor.updateIframe();
            if (Prototype.Browser.IE) {
                var rng = editor.iframe.contentWindow.document.body.createTextRange();
                rng.findText('...pasting...');
                rng.select();
            }
            else {
                editor.iframe.contentWindow.getSelection().selectAllChildren(editor.iframe.contentWindow.document.getElementById("caret"));
            }
            var caret = editor.iframe.contentWindow.document.getElementById("caret");
            caret.parentNode.removeChild(caret);

        }, 1); // execute after paste
    }
};

Event.observe(window, 'load', function () {
    var editors = $$('.PURichText');

    editors.each(function (textarea) {
        var id = textarea.id;
        var editor = new PURichTextEditor(textarea);
        PURichText.editors.set(id, editor);
        if (Prototype.Browser.IE) {
            var onPaste = function () { PURichText.onPasteHandler(id); }
            Event.observe(editor.iframe.contentWindow.document.body, 'paste', onPaste);
        }
    });

})

// class level functions
var PURichTextEditor = Class.create({

    // This array holds the stylesheets for editor windows.
    mobileBrowser: false,
    // Create an array of hash groups with the default tools that use execCommand
    tools: [
		$H({
		    bold: Util.Translate('BOLD'),
		    italic: Util.Translate('ITALIC'),
		    underline: Util.Translate('UNDERLINE'),
		    justifyleft: Util.Translate('ALIGN_LEFT'),
		    justifycenter: Util.Translate('ALIGN_CENTER'),
		    justifyright: Util.Translate('ALIGN_RIGHT'),
		    insertorderedlist: Util.Translate('ORDERED_LIST'),
		    insertunorderedlist: Util.Translate('BULLETPOINT_LIST'),
		    outdent: Util.Translate('OUTDENT'),
		    indent: Util.Translate('INDENT'),
		    undo: Util.Translate('UNDO'),
		    redo: Util.Translate('REDO'),
		    toggleSource: Util.Translate('EDIT_HTML_SOURCE')
		})
	],

    // Tools for HTML editing
    sourceTools: $H({
        toggleSource: Util.Translate('BACK_TO_EDITOR')
    }),

    initialize: function (textarea) {

        this.textarea = textarea;
        // not a complete list
        var mobileUserAgentRe = new RegExp('(Mobile|BlackBerry|HTC)');
        if (navigator.userAgent.match(mobileUserAgentRe)) {
            // iPad - set PlainText input to true and show border
            this.mobileBrowser = true;
            textarea.next().value = 'true';
            textarea.setStyle({
                'border': '1px solid #C6C6C6 !important',
                'width': '99.5%'
            });

            var HTMLText = textarea.value;
            // replace these closing tags with 2 new lines
            var newLineReplacesments = new RegExp('</\s?(p|ul|ol|h1|h2|h3|h4|h5|table)>', 'gi');
            HTMLText = HTMLText.replace(newLineReplacesments, '{NEWLINE}{NEWLINE}');

            // replace these tags with 1 new line
            var newLineReplacements = new RegExp('<(/li|br ?/|/tr)>', 'gi');
            HTMLText = HTMLText.replace(newLineReplacements, '{NEWLINE}');

            // Remove every other HTML tag :( bye bye
            var tagStripper = new RegExp('</?(.*?)>', 'gi');
            HTMLText = HTMLText.replace(tagStripper, '');

            // return new lines
            var newLineReplacer = new RegExp('{NEWLINE}', 'gi');
            HTMLText = HTMLText.replace(newLineReplacer, '\r\n');

            // put content in textarea
            textarea.value = HTMLText;

            // Put textarea in a wrapper div
            var wrapper = new Element('div', {
                id: textarea.id + '_wrapper'
            }).addClassName('PURichTextWrapper').setStyle({
                'width': wrapperWidth,
                'border': 'none !important'
            });

            textarea.parentNode.replaceChild(wrapper, textarea);
            wrapper.appendChild(textarea);
        }
        else {
            this.id = textarea.getAttribute('id');
            this.toolbar = null;
            this.height = textarea.getHeight();
            this.width = textarea.getWidth();

            // IE has some problems calculating dimensions depending on styles of parent elements
            if (this.height < 50) {
                this.height = 200;
                this.textarea.setStyle({ 'height': '200px' })
            }
            var wrapperWidth = this.width + 'px';
            if (this.width <= 0)
                wrapperWidth = '100%';

            // Get form that is holding the textarea, and setup submit listener
            this.form = textarea.up('form');
            if (this.form != undefined) {
                this.form.observe('submit', this.submitHandler.bind(this));
            }

            // Array to hold our elements to add into the document
            var elements = [];

            // To create reference to a function given a subject and a name of the function as a String.
            var makeHandler = function (subject, method) {
                return function () { subject[method](); }
            }

            // Create Toolbar
            this.toolbar = new Element('ul', {
                id: this.id + '_toolbar'
            }).addClassName('rt_toolbar');

            // Within prototype's iterators you lose reference to current object
            var self = this;

            // Add Editor Tools
            this.tools.each(function (toolGroup) {
                toolGroup.each(function (tool) {
                    var lili = self.toolbar.appendChild(new Element('li'));
                    var a = lili.appendChild(new Element('a', {
                        title: tool.value
                    }).addClassName(tool.key).update(tool.value));
                    if (!Prototype.Browser.IE) {

                        var clickHandler = function () { self['editorCommand'](tool.key, ''); };
                        a.observe('click', clickHandler);
                    }
                    else {
                        /* IE loses the selection object when you click elsewhere - do it the Bad Old Way
                        Other browsers keep selections for each window */
                        a.href = 'javascript:PURichText.editors.get("' + self.id + '").editorCommand("' + tool.key + '", "")';
                    }
                });
            });
            elements.push(this.toolbar);

            // Add View Source Tools
            this.srcToolbar = new Element('ul', {
                id: this.id + '_source_toolbar'
            }).addClassName('rt_source_toolbar').setStyle({
                display: 'none'
            });
            elements.push(this.srcToolbar);

            this.sourceTools.each(function (tool) {
                var li = self.srcToolbar.appendChild(new Element('li'));
                var a = li.appendChild(new Element('a', {
                    title: tool.value
                }).addClassName(tool.key));
                a.update(tool.value).observe('click', makeHandler(self, tool.key));
            });

            // Create iframe of same dimensions as textarea
            this.iframe = new Element('iframe', {
                id: this.id + '_iframe',
                name: this.id + '_iframe',
                frameBorder: 0,
                src: 'javascript:false'
            }).addClassName('rt_iframe').setStyle({
                height: this.height + 'px',
                width: '100%'
            });
            elements.push(this.iframe);

            // Put textarea in a wrapper div
            var wrapper = new Element('div', {
                id: this.id + '_wrapper'
            }).addClassName('PURichTextWrapper');

            textarea.parentNode.replaceChild(wrapper, textarea);

            // Add toolbar and iframe before textarea
            elements.each(function (element) {
                wrapper.insert(element);
            });

            // Add textarea last
            wrapper.appendChild(textarea);

            this.textarea.hide();
            this.contentWindow = this.iframe.contentWindow;
            if (Prototype.Browser.Gecko) {
                Event.observe(this.contentWindow, 'keyup', this.editorKeyUpHandler.bind(this));
            }
            else if (Prototype.Browser.WebKit) {
                Event.observe(this.contentWindow.document, 'keyup', this.editorKeyUpHandler.bind(this));
            }

            var resizer = new Element('div', {
                id: this.id + '_resizer'
            }).addClassName('PURichTextResizer');

            wrapper.appendChild(resizer);

            new Draggable(resizer, {
                constraint: 'vertical',
                onStart: this.startResize,
                onEnd: this.resize.bind(this)
            });

            this.createDocument();

            // enable the designMode	
            try {
                if (Prototype.Browser.IE) {
                    this.iframe.contentWindow.document.body.unselectable = "on";
                    this.iframe.contentWindow.document.body.contentEditable = 'true';
                    setTimeout("PURichText.editors.get('" + this.id + "').iframe.contentWindow.document.body.removeAttribute('unselectable');", 1);
                }
                else {
                    this.contentWindow.document.designMode = 'on';
                }
            }
            catch (e) { /* alert(e); */ }

            // FF's 'styledWithCSS' html is bloated
            this.editorCommand('useCSS', false);
            this.editorCommand('styleWithCSS', false);
        }
    },
    startResize: function () {
        $('underlay').show();
    },
    resize: function (effect) {
        try {
            var height = Number(effect.element.style.top.replace('px', '')) + $(effect.element).up('div').getHeight();
            if (height < 100) { height = 100 };
        }
        catch (err) { var height = 100; }
        this.iframe.setStyle({ height: (height - 40) + 'px' });
        this.textarea.setStyle({ height: (height - 40) + 'px' })
        this.contentWindow.document.body.style.height = (height - 65) + 'px';
        effect.element.style.top = 0;
        $('underlay').hide();
    },
    createDocument: function () {

        // For Prototypes iterator loss of 'this' reference
        var cwd = this.contentWindow.document;
        var iframeStylesheets = [Page.ROOT_DIR + 'richie-iframe.css'];
        // write the content of textarea to the iframe, an embedded stylesheet was all that worked in multiple browsers..
        cwd.open();

        cwd.write('<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd"><html><head>');
        cwd.write('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />');
        // Add style sheets
        cwd.write('<style type="text/css">');
        if (iframeStylesheets.length > 0) {
            iframeStylesheets.each(function (ss) {
                cwd.write('@import url(' + ss + ');');
            });
        }
        cwd.write('</style>');
        cwd.write('</head><body class="editor_body" style="height: ' + (this.height - 30) + 'px; width: ' + (this.width - 40) + 'px"></body></html>');
        this.contentWindow.document.body.innerHTML = this.textarea.value;
        cwd.close();
    },

    // assign the value of the iframe to the textarea
    updateTextarea: function () {
        this.textarea.value = this.contentWindow.document.body.innerHTML;
    },

    // assign the value of the textarea to the iframe
    updateIframe: function () {
        this.contentWindow.document.body.innerHTML = this.textarea.value;
    },

    // View Source
    toggleSource: function () {

        // Hide visible editor and show hidden editor
        (this.textarea.visible()) ? this.updateIframe() : this.updateTextarea();
        this.textarea.toggle();
        this.srcToolbar.toggle();
        this.toolbar.toggle();
        this.iframe.toggle();
        if (this.textarea.visible()) {
            this.textarea.focus();
        }
    },

    /* A format from this.formatOptions has been selected */
    formatSelection: function () {
        this.editorCommand('formatblock', '<' + this.formatSelect.options[this.formatSelect.selectedIndex].value + '>');
        this.formatSelect.selectedIndex = 0;
    },

    // Uses insertHTML execCommand to put HTML into the editor at the point of selection
    addContent: function (content) {
        if (this.textarea.visible()) {
            this.toggleSource();
        }
        this.contentWindow.focus();

        // IE doesn't have an insertHTML command
        if (Prototype.Browser.IE) {
            var range = this.contentWindow.document.selection.createRange();
            range.pasteHTML(content);
            range.collapse(false);
            range.select();
        }
        else {
            this.editorCommand('insertHTML', content);
        }
    },

    // replaces HTML in textarea and iframe with value
    setContent: function (content) {
        this.contentWindow.focus();
        this.textarea.value = content;
        this.updateIframe();
    },

    // updates content in textarea and returns HTML
    getContent: function () {
        if (!this.textarea.visible())
            this.updateTextarea();
        return this.textarea.value;
    },

    /** Calls execCommand function - various levels of support in browsers and different html output
    * Helpful links regarding designMode and execCommand
    * http://msdn2.microsoft.com/en-us/library/ms533049(VS.85).aspx
    * http://www.mozilla.org/editor/ie2midas.html 
    **/
    editorCommand: function (command, options) {
        if (command == 'toggleSource') {
            this.toggleSource();
        }
        try {
            this.contentWindow.focus();
            this.contentWindow.document.execCommand(command, false, options);
            this.contentWindow.focus();
        } catch (e) { /* alert(e); */ }
    },

    /* Range and Selection objects are created and used to get information about the currently 
    * selected content in a document, IE uses a different implementation to other browsers.
    **/
    getSelection: function () {
        return (window.getSelection) ? this.contentWindow.getSelection() : this.contentWindow.document.selection.createRange();
    },

    /* Returns the block that is in the currect Selection */
    getSelectedContainer: function () {

        var s = this.getSelection();
        if (s.parentElement && s.parentElement != undefined) {
            elm = s.parentElement();
            /* IE loses selection focus when you click outside the editor so ensure the selection 
            * is within a text editor. A bit of a hack to see where the selection is */
            try {
                var test = elm.up('');
            }
            catch (e) { return elm; }
            return false;
        }
        else {
            var node = s.focusNode;
            if (!node)
                return false;
            if (node.nodeName == '#text') {
                return node.parentNode;
            }
            else {
                return node;
            }
        }
        return false;
    },

    /* Do cleanup of editor before form data */
    submitHandler: function (event) {
        if (this.mobileBrowser) {
            this.mobileBrowserCleanHTML();
        }
        else {
            if (this.iframe.visible())
                this.updateTextarea();

            this.cleanHTML();
        }
        // Show content in textarea and stop Submit - For testing purposes
        //alert(this.textarea.value);
        //Event.stop(event);
    },

    // Cleans up generated markup - makes ff closer to IE's rendering
    editorKeyUpHandler: function (e) {

        var key = e.which || e.keyCode;
        var elm = this.getSelectedContainer();
        var name = elm.tagName.toLowerCase();
        var shift = e.shiftKey;
        if (e.ctrlKey && key == 86) {
            PURichText.onPasteHandler(this.id);
        }
        else {
            switch (key) {
                case Event.KEY_RETURN:
                    if (!shift) {
                        // Cleanup <br><br>'s in ff
                        var nodes = this.contentWindow.document.body.childNodes;
                        for (var i = 0; i < nodes.length; i++) {
                            if (nodes.item(i).nodeName.toLowerCase() == 'br') {
                                this.contentWindow.document.body.removeChild(nodes.item(i));
                            }
                        }
                    }
                    var inlineTags = ['strong', 'b', 'em', 'i', 'sub', 'sup', 'a'];
                    if (inlineTags.indexOf(name) != -1)
                        name = elm.parentNode.tagName.toLowerCase();
                    if (name == 'body')
                        this.editorCommand('formatblock', '<P>');
                    break;
                case 17:
                    break;
                default:
                    if (name == 'body') {
                        // make stray elements paragraphs
                        this.editorCommand('formatblock', '<P>');
                    }
                    break;
            }
        }
    },

    // turn new line characters into <br> 's
    mobileBrowserCleanHTML: function () {
        var input = this.textarea.value;
        // 1. remove line breaks
        var lineBreakStripper = /(\n)/g;
        var output = input.replace(lineBreakStripper, '<br />');

        this.textarea.value = output;
    },

    // removes MS Office generated guff
    cleanHTML: function () {

        var input = this.textarea.value;
        // 1. remove line breaks / Mso classes
        var lineBreakStripper = /(\n)/g;
        var output = input.replace(lineBreakStripper, '{NEWLINE}');
        lineBreakStripper = /(\r)/g;
        output = output.replace(lineBreakStripper, '{CRETURN}');

        var stringStripper = /( class=(")?Mso[a-zA-Z0-9]+(")?)/g;
        output = output.replace(stringStripper, '');
        // 2. strip Word generated HTML comments
        var commentSripper = new RegExp('<!--(.*?)-->', 'g');
        output = output.replace(commentSripper, '');
        var tagStripper = new RegExp('<(/)*(meta|link|span|\\?xml:|st1:|o:|font)(.*?)>', 'gi');
        // 3. remove tags leave content if any
        output = output.replace(tagStripper, '');
        // 4. Remove everything in between and including tags '<style(.)style(.)>'
        var badTags = ['style', 'script'];
        for (var i = 0; i < badTags.length; i++) {
            tagStripper = new RegExp('<' + badTags[i] + '.*?' + badTags[i] + '(.*?)>', 'gi');
            output = output.replace(tagStripper, '');
        }

        output = output.gsub('{NEWLINE}', '\n');
        output = output.gsub('{CRETURN}', '\r');
        this.textarea.value = output;
    }

});